@@ -3,8 +3,12 @@ source('test_functions.R')
 
 warn <- options('warn')
 options(warn=2)
-#test.prediction()
 test.expressions()
 
+# longer tests (comment out for submission)
+#test.prediction()
+#test.prediction.with.prob.migration()
 #test.expressions.with.VE(map=FALSE)
-try(options(warn=warn), silent=TRUE)
\ No newline at end of file
+#test.regional.aggregation()
+
+options(warn=warn$warn)
\ No newline at end of file
